@@ -106,6 +106,7 @@ bonobo_ui_node_copy (BonoboUINode *node,
 				child->prev = last;
 				last->next  = child;
 			}
+			last = child;
 		}
 	}
 
@@ -128,14 +129,12 @@ bonobo_ui_node_add_child (BonoboUINode *parent,
 	for (l = parent->children; l; l = l->next)
 		last = l;
 
-	if (!last) {
+	child->prev = last;
+	child->next = NULL;
+	if (!last)
 		parent->children = child;
-		child->prev = NULL;
-		child->next = NULL;
-	} else {
+	else
 		last->next  = child;
-		child->prev = last;
-	}
 	child->parent = parent;
 }
 
@@ -302,6 +301,7 @@ BonoboUINode *
 bonobo_ui_node_ref (BonoboUINode *node)
 {
 	node->ref_count++;
+	return node;
 }
 
 /**
@@ -1107,6 +1107,8 @@ bonobo_ui_node_to_string (BonoboUINode *node,
 {
 	GString *str = g_string_sized_new (64);
 
+	g_return_val_if_fail (node != NULL, NULL);
+
 	internal_to_string (str, node, recurse);
 
 /*	fprintf (stderr, "nodes to string: '%s'", str->str); */
