@@ -13,6 +13,8 @@
 #include <glib/gmacros.h>
 #include <gtk/gtk.h>
 
+#ifndef BONOBO_UI_DISABLE_DEPRECATED
+
 G_BEGIN_DECLS
 
 #define BONOBO_TYPE_UI_TOOLBAR_ITEM			(bonobo_ui_toolbar_item_get_type ())
@@ -95,4 +97,6 @@ void                      bonobo_ui_toolbar_item_activate         (BonoboUIToolb
 
 G_END_DECLS
 
+#endif
+
 #endif /* __BONOBO_UI_TOOLBAR_ITEM_H__ */
